"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const enhance_1 = require("./enhance");
const utils_1 = require("./utils");
class Collection {
    root;
    constructor(root) {
        this.root = root;
    }
    /**
     * Retrieve a locator to a given element on the page identified by the
     * selector. The locator is lazily initialized when retrieved to ensure that
     * the most current `root` element is used.
     *
     * If `options.portal` is set to true, the locator will be based off the
     * `frame`, rather than the `root` thus escaping from any collection nesting.
     * This is useful to represent a page structure whose visual appearance
     * differs from it's DOM structure.
     *
     * @param selector - The selector that identifies the element.
     * @param options - Options for how to build the locator.
     */
    el(selector, options) {
        return options?.portal
            ? this.frame.locator(selector)
            : this.root.locator(selector);
    }
    nest(collection, root) {
        const rootElement = typeof root === 'string' ? this.el(root) : root;
        const instance = new collection(rootElement);
        // If the root element is a locator, it can use the `nth`, `first`, and
        // `last` methods and thus it should be enhanced.
        return utils_1.isLocator(rootElement)
            ? enhance_1.enhance(collection, rootElement, instance)
            : instance;
    }
    /**
     * Returns the page or frame that the collection is attached to. This can be
     * used when nesting a collection if the nested collection's locators elements
     * should be based off the page or frame rather than the parent collection's
     * root.
     *
     * @example this.nest(TextField, this.frame)
     */
    get frame() {
        return utils_1.isLocator(this.root)
            ? // Playwright doesn't currently expose the frame of a locator as a
                // public API, so for now we need to get the private property.
                this.root._frame
            : this.root;
    }
    /**
     * Returns the page that the collection is attached to. This can be used when
     * you need to access page methods inside your collection utility methods. For
     * example, if a utility needs to access `page.mouse`, this will allow that.
     *
     * @example this.page.mouse.down()
     */
    get page() {
        return 'page' in this.frame ? this.frame.page() : this.frame;
    }
}
exports.Collection = Collection;
//# sourceMappingURL=Collection.js.map